<?php

namespace EventPortal\Domain\Component\Concerns;

use Illuminate\Database\Eloquent\Collection;

trait HasOptions
{
    /** @var array<\EventPortal\Domain\Component\Option> */
    public $options = [];

    public function hasOptions(): bool
    {
        return true;
    }

    public function options(?array $options): self
    {
        $this->options = $this->convertToArray($options);

        return $this;
    }

    /**
     * @param mixed $value
     * @return array|mixed|null
     */
    public function getOptions($value)
    {
        if ($value === null || is_array($value)) {
            return $value;
        }

        if (is_bool($value) || is_int($value)) {
            return collect($this->options)->where('value', $value)->first();
        }

        if (get_class($value) === Collection::class) {
            return collect($this->options)->whereIn('value', $value->pluck('id'))->values();
        }

        return $value;
    }

    /**
     * @return mixed
     */
    public function getValue()
    {
        return json_decode($this->value);
    }

    /**
     * Map array of Option into plain multi-dimensional array.
     */
    private function convertToArray(array $options): array
    {
        return collect($options)->map(function ($options): array {
            if (is_array($options)) {
                return $options;
            }

            return $options->toArray();
        })
        ->toArray();
    }
}
